/* Initialize DLL entry point etc.

   I've used DllMain as the entry point since that's the most common 
   usage (MSVC and Mingw32 default to DllMain and Borland defaults to 
   DllEntryPoint). Cygwin32 doesn't really have a (working) entry point
   for user DLLs, so you must supply the entry point flag -e,_DllMain@12
   to the linker or -Wl,-e,_DllMain@12 via the language driver such as
   gcc or c++. Note that you must specify _DllMain@12, not just DllMain.

   Mingw32 notes: Note the calls to __do_global_ctors and __do_global_dtors 
   in DllMain. These may or may not be needed depending on your version
   of libmingw32.a, so leave them in if you have problems with global
   C++ construction/destruction in your DLL.

   Cygwin32 notes: Making C++ user DLLs in cygwin32 can be somewhat of a
   black magic. Just leave the code as is in this file, and you should
   be ok. Also, don't do anything else in DLL_PROCESS_ATTACH or your
   code may just crash deep within cygwin dll; for example, sticking a
   printf statement in DLL_PROCESS_ATTACH can cause a crash.

 */


#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#undef WIN32_LEAN_AND_MEAN
#include <stdio.h>

#ifdef __CYGWIN32__
/* cygwin32 requires an impure pointer variable, which must be
   explicitly initialized when the DLL starts up.  */
struct _reent *_impure_ptr;
extern struct _reent *__imp_reent_data;

/* DLL C++ global initializers from libgcc2.c */

/*  Declare a pointer to void function type.  */
typedef void (*func_ptr) (void);

/* Declare the set of symbols use as begin and end markers for the lists
   of global object constructors and global object destructors.  */

extern func_ptr __CTOR_LIST__[];
extern func_ptr __DTOR_LIST__[];

void
__do_global_dtors ()
{
  static func_ptr *p = __DTOR_LIST__ + 1;
  while (*p)
    {
      p++;
      (*(p - 1)) ();
    }
}

void
__do_global_ctors ()
{
  unsigned long nptrs = (unsigned long) __CTOR_LIST__[0];
  unsigned i;
  if (nptrs == (unsigned long) -1)
    for (nptrs = 0; __CTOR_LIST__[nptrs + 1] != 0; nptrs++);
  for (i = nptrs; i >= 1; i--)
    __CTOR_LIST__[i] ();
}

#endif /* __CYGWIN32__ */

#ifdef __MINGW32__
extern void __do_global_dtors ();
extern void __do_global_ctors ();
#endif /* __MINGW32__ */

BOOL APIENTRY DllMain (HINSTANCE hInst, DWORD reason, 
                       LPVOID reserved /* Not used. */ );

/*
 *----------------------------------------------------------------------
 *
 * DllMain --
 *
 *      This routine is called by the VC++ C run time library init
 *      code, or the DllEntryPoint routine.  It is responsible for
 *      initializing various dynamically loaded libraries.
 *
 * Results:
 *      TRUE on sucess, FALSE on failure.
 *
 * Side effects:
 *      Establishes 32-to-16 bit thunk and initializes sockets library.
 *
 *----------------------------------------------------------------------
 */
BOOL APIENTRY
DllMain (
	 HINSTANCE hInst /* Library instance handle. */ ,
	 DWORD reason /* Reason this function is being called. */ ,
	 LPVOID reserved /* Not used. */ )
{
#ifdef __CYGWIN32__
  /* cygwin32 requires the impure data pointer to be initialized
     when the DLL starts up.  */
  _impure_ptr = __imp_reent_data;
#endif /* __CYGWIN32__ */

  switch (reason)
    {
    case DLL_PROCESS_ATTACH:
#ifdef __GNUC__
      __do_global_ctors ();
#endif /* __GNUC__ */
      return TRUE;

    case DLL_PROCESS_DETACH:
#ifdef __GNUC__
      __do_global_dtors ();
#endif /* __GNUC__ */
      break;
    }
  return TRUE;
}

